@@ -2,13 +2,14 @@
 Pages for sweeping uncertainty propagation and reverse propagations.
 '''
 from contextlib import suppress
+import re
 import numpy as np
 from PyQt5 import QtWidgets, QtGui, QtCore
+from pint import DimensionalityError, UndefinedUnitError, OffsetUnitCalculusError
 
-from .. import sweeper
-from . import gui_common
+from . import gui_common  # noqa: F401
 from . import gui_widgets
-from . import page_uncertprop
+from . import page_uncert
 from . import page_reverse
 from . import page_dataimport
 
@@ -17,6 +18,9 @@ class StartStopCountWidget(QtWidgets.QDialog):
     ''' Dialog for defining a sweep from start/stop/step values '''
     def __init__(self, title=None, parent=None):
         super().__init__(parent=parent)
+        font = self.font()
+        font.setPointSize(10)
+        self.setFont(font)
         self.setWindowTitle(title if title else 'Enter Range')
         self.start = QtWidgets.QLineEdit()
         self.stop = QtWidgets.QLineEdit()
@@ -26,7 +30,7 @@ class StartStopCountWidget(QtWidgets.QDialog):
         validator.setNotation(QtGui.QDoubleValidator.StandardNotation | QtGui.QDoubleValidator.ScientificNotation)
         self.start.setValidator(validator)
         self.stop.setValidator(validator)
-        self.num.setValidator(QtGui.QIntValidator(0, 1E5))
+        self.num.setValidator(QtGui.QIntValidator(0, 10000))
         flayout = QtWidgets.QFormLayout()
         flayout.addRow('Start:', self.start)
         flayout.addRow('Stop:', self.stop)
@@ -41,50 +45,48 @@ class StartStopCountWidget(QtWidgets.QDialog):
     def get_range(self):
         ''' Get array of values. Blank array if not all params defined. '''
         try:
-            return np.linspace(float(self.start.text()), float(self.stop.text()), float(self.num.text()))
-        except ValueError:
+            return np.linspace(float(self.start.text()), float(self.stop.text()), int(self.num.text()))
+        except (TypeError, ValueError):
             return np.array([])
 
 
 class SweepSetupTable(gui_widgets.FloatTableWidget):
     ''' Table for defining sweep variables '''
-    def __init__(self, calc=None, parent=None):
+    def __init__(self, projitem=None, parent=None):
         super().__init__(parent=parent, movebyrows=True, paste_multicol=False)
-        self.sweepcalc = calc
-        self.inptlist = []
-        self.setStyleSheet(page_uncertprop.TABLESTYLE)
-        
-        # This is stupid and ugly but otherwise the keyboard navigation doesn't work with a table 
+        self.projitem = projitem
+        self.variables = None
+        self.setStyleSheet(page_uncert.page_uncert_input.TABLESTYLE)
+
+        # This is stupid and ugly but otherwise the keyboard navigation doesn't work with a table
         # embedded in a tree. It works fine when table isn't embedded in tree.
         self.setEditTriggers(QtWidgets.QTableWidget.AllEditTriggers)
 
         # Load initial values from saved file
-        self.setColumnCount(len(self.sweepcalc.sweeplist))
-        for swpidx, swp in enumerate(self.sweepcalc.sweeplist):
+        self.setColumnCount(len(self.projitem.model.sweeplist))
+        for swpidx, swp in enumerate(self.projitem.model.sweeplist):
             values = swp.get('values', [])
             if len(values) > self.rowCount():
                 self.setRowCount(len(values))
 
             for i, val in enumerate(values):
                 self.setItem(i, swpidx, QtWidgets.QTableWidgetItem(str(val)))
-            inptname = swp.get('inptname')
+            var = swp.get('var')
             comp = swp.get('comp')
             param = swp.get('param')
-            if inptname == 'corr':
-                sweepstr = 'corr({}, {})'.format(swp.get('var1'), swp.get('var2'))
+            if var == 'corr':
+                sweepstr = f'corr({swp.get("var1")}, {swp.get("var2")})'
             elif comp == 'nom':
-                sweepstr = 'Mean({})'.format(swp.get('var'))
+                sweepstr = f'Mean({swp.get("var")})'
             elif param == 'df':
-                sweepstr = 'df({})'.format(comp)
+                sweepstr = f'df({comp})'
             else:
-                sweepstr = 'Unc({}, {}, {})'.format(swp.get('var'), comp, param)
+                sweepstr = f'Unc({swp.get("var")}, {comp}, {param})'
             self.setHorizontalHeaderItem(swpidx, QtWidgets.QTableWidgetItem(sweepstr))
 
-        self.valueChanged.connect(self.update_sweepparams)
-
-    def set_inptlist(self, inptlist):
-        ''' Set the list of inputs '''
-        self.inptlist = inptlist
+    def set_variables(self, variables):
+        ''' Set the Variables object '''
+        self.variables = variables
 
     def clear(self):
         ''' Clear the table '''
@@ -123,11 +125,10 @@ class SweepSetupTable(gui_widgets.FloatTableWidget):
             self.blockSignals(True)
             self.removeColumn(col)
             self.blockSignals(False)
-            self.update_sweepparams()
 
     def addcol(self):
         ''' Add a column to the sweep table '''
-        w = SweepParamWidget(self.inptlist)
+        w = SweepParamWidget(self.variables)
         status = w.exec_()
         if status:
             self.blockSignals(True)
@@ -150,10 +151,10 @@ class SweepSetupTable(gui_widgets.FloatTableWidget):
         elif col < 0:
             col = 0
 
-        dlg = page_dataimport.ArraySelectWidget(singlecol=True, project=self.sweepcalc.project)
+        dlg = page_dataimport.ArraySelectWidget(singlecol=True, project=self.projitem.project)
         ok = dlg.exec_()
         if ok:
-            sweep = dlg.get_array().y
+            sweep = dlg.get_array().get('y')
             if sweep is not None:
                 self.blockSignals(True)
                 if len(sweep) > self.rowCount():
@@ -167,7 +168,7 @@ class SweepSetupTable(gui_widgets.FloatTableWidget):
                 for i, val in enumerate(sweep):
                     self.item(i, col).setText(str(val))
                 self.blockSignals(False)
-                self.update_sweepparams()
+                self.resizeColumnsToContents()
 
     def filldata(self):
         ''' Fill selected column with start/stop/step values '''
@@ -198,47 +199,42 @@ class SweepSetupTable(gui_widgets.FloatTableWidget):
             for i, val in enumerate(rng):
                 self.item(i, col).setText(str(val))
             self.blockSignals(False)
-            self.update_sweepparams()
 
-
-    def update_sweepparams(self):
-        ''' Update UncertSweep model with currently displayed parameters '''
+    def get_config(self):
+        ''' Get sweep configuration dictionary '''
         sweeplist = []
         for col in range(self.columnCount()):
             sweepstr = self.horizontalHeaderItem(col).text()
-            vals = [self.item(i, col).text() for i in range(self.rowCount()) if self.item(i, col) and self.item(i, col).text() != '']
+            vals = [self.item(i, col).text()
+                    for i in range(self.rowCount()) if self.item(i, col) and self.item(i, col).text() != '']
             try:
                 vals = np.array([float(v) for v in vals])
             except ValueError:
                 return
             start = sweepstr.find('(')
             params = sweepstr[start+1:-1].split(', ')
-            d = {'mode': sweepstr[:start],
-                 'values': vals,
-                 'varname': params[0]}
-            if len(params) > 1:
-                d['comp'] = params[1]
-            if len(params) > 2:
-                d['param'] = params[2]
+            mode = sweepstr[:start]
+            if mode == 'Mean':
+                d = {'var': params[0], 'comp': 'nom'}
+            elif mode == 'Unc':
+                d = {'var': params[0], 'comp': params[1], 'param': params[2]}
+            elif mode == 'df':
+                d = {'var': params[0], 'param': 'df'}
+            elif mode == 'corr':
+                d = {'var': 'corr', 'var1': params[0], 'var2': params[1]}
+            d['values'] = vals
             sweeplist.append(d)
-
-        self.sweepcalc.clear_sweeps()
-        for params in sweeplist:
-            if params['mode'] == 'Mean':
-                self.sweepcalc.add_sweep_nom(params['varname'], params['values'])
-            elif params['mode'] == 'Unc':
-                self.sweepcalc.add_sweep_unc(params['varname'], params['values'], params['comp'], params['param'])
-            elif params['mode'] == 'df':
-                self.sweepcalc.add_sweep_df(params['varname'], params['values'])
-            elif params['mode'] == 'corr':
-                self.sweepcalc.add_sweep_corr(params['varname'], params['comp'], params['values'])
+        return sweeplist
 
 
 class SweepParamWidget(QtWidgets.QDialog):
     ''' Widget for selecting what is being swept in this column '''
-    def __init__(self, inptlist, parent=None):
+    def __init__(self, variables, parent=None):
         super().__init__(parent=parent)
-        self.inptlist = inptlist
+        font = self.font()
+        font.setPointSize(10)
+        self.setFont(font)
+        self.variables = variables
         self.setWindowTitle('Add Parameter Sweep')
         self.mode = QtWidgets.QComboBox()
         self.mode.addItems(['Mean', 'Uncertainty', 'Degrees of Freedom', 'Correlation'])
@@ -256,8 +252,8 @@ class SweepParamWidget(QtWidgets.QDialog):
         self.buttons = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)
         self.buttons.rejected.connect(self.reject)
         self.buttons.accepted.connect(self.accept)
-        self.varname.addItems([v.name for v in inptlist])
-        self.var2.addItems([v.name for v in inptlist])
+        self.varname.addItems(self.variables.names)
+        self.var2.addItems(self.variables.names)
 
         unclayout = QtWidgets.QHBoxLayout()
         unclayout.addWidget(self.lblcomp)
@@ -292,40 +288,41 @@ class SweepParamWidget(QtWidgets.QDialog):
         ''' Variable selection has changed '''
         self.blockSignals(True)
         name = self.varname.currentText()
-        varnames = [v.name for v in self.inptlist]
-        idx = varnames.index(name)
-        comps = [u.name for u in self.inptlist[idx].uncerts]
+
+        comps = self.variables.get(name).typeb_names
         self.unccomp.clear()
         if len(comps) > 0:
             self.unccomp.addItems(comps)
         else:
-            self.unccomp.addItems(['u({})'.format(name)])
+            self.unccomp.addItems([f'u({name})'])
         self.blockSignals(False)
 
     def compchanged(self):
         ''' Component selection changed '''
         self.blockSignals(True)
         varname = self.varname.currentText()
-        varidx = [v.name for v in self.inptlist].index(varname)
         compname = self.unccomp.currentText()
         if compname:
-            compidx = [u.name for u in self.inptlist[varidx].uncerts].index(compname)
+            typeb = self.variables.get(varname).get_typeb(compname)
+            if typeb.distname in ['normal', 't']:
+                items = ['unc', 'k']
+            else:
+                items = typeb.required_args
             self.uncparam.clear()
-            self.uncparam.addItems(self.inptlist[varidx].uncerts[compidx].args.keys())
+            self.uncparam.addItems(items)
         self.blockSignals(False)
 
     def get_sweepstr(self):
         ''' Get string representation of sweep '''
         if self.mode.currentText() == 'Mean':
-            return 'Mean({})'.format(self.varname.currentText())
+            return f'Mean({self.varname.currentText()})'
         elif self.mode.currentText() == 'Degrees of Freedom':
-            return 'df({})'.format(self.varname.currentText())
+            return f'df({self.varname.currentText()})'
         elif self.mode.currentText() == 'Correlation':
-            return 'corr({}, {})'.format(self.varname.currentText(), self.var2.currentText())
+            return f'corr({self.varname.currentText()}, {self.var2.currentText()})'
         elif self.mode.currentText() == 'Uncertainty':
-            return 'Unc({}, {}, {})'.format(self.varname.currentText(), self.unccomp.currentText(), self.uncparam.currentText())
-        else:
-            assert False
+            return f'Unc({self.varname.currentText()}, {self.unccomp.currentText()}, {self.uncparam.currentText()})'
+        raise ValueError
 
     def modechange(self):
         ''' Mode (mean, uncertainty, correlation, etc.) of sweep has changed '''
@@ -380,32 +377,30 @@ class SweepSlider(QtWidgets.QWidget):
         return getattr(self.slider, name)
 
 
-class PageOutputSweep(page_uncertprop.PageOutput):
+class PageOutputSweep(page_uncert.page_uncert_output.PageOutput):
     ''' Page for viewing output of sweep. Adds a slider to select
         individual runs in the sweep
     '''
-    def __init__(self, sweepReport=None, parent=None):
-        super().__init__(uncCalc=None, parent=parent)
-        self.sweepReport = sweepReport
+    def __init__(self, sweepresult=None, parent=None):
+        super().__init__(parent=parent)
+        self.sweepresult = sweepresult
         self.slider = SweepSlider()
         self.slider.valueChanged.connect(self.sliderchange)
         self.oplayout.insertWidget(1, self.slider)  # Add just below combobox
 
-    def set_sweepreport(self, sweepreport):
+    def set_sweepresult(self, sweepresult):
         ''' Set the report to show '''
-        self.sweepReport = sweepreport
-        self.slider.setRange(0, self.sweepReport.N-1)
+        self.sweepresult = sweepresult
+        self.slider.setRange(0, len(self.sweepresult)-1)
         self.slider.setValue(0)
-        self.update(self.sweepReport.outputlist[0])
-        self.slider.setLblTxt(self.sweepReport.get_single_desc(0))
+        self.update(self.sweepresult[0])
+        self.slider.setLblTxt(self.sweepresult.report.describe(0))
 
     def sliderchange(self):
         ''' Slider was changed, update the unccalc report '''
         i = self.slider.value()
-        self.set_unccalc(self.sweepReport.outputlist[i].ucalc)
-        self.update(self.sweepReport.outputlist[i])
-        # TODO: update slider to show math image instead of text only, get_single_desc would return latex
-        self.slider.setLblTxt(self.sweepReport.get_single_desc(i))
+        self.update(self.sweepresult[i])
+        self.slider.setLblTxt(self.sweepresult.report.describe(i))
         self.outputupdate()
 
     def outputupdate(self):
@@ -414,8 +409,8 @@ class PageOutputSweep(page_uncertprop.PageOutput):
         '''
         option = self.outputSelect.currentText()
         if option == 'Summary':
-            r = self.sweepReport.report_summary(uy=':')
-            self.txtOutput.setReport(r)
+            rpt = self.sweepresult.report.summary_withplots()
+            self.txtOutput.setReport(rpt)
             self.outputStack.setCurrentIndex(1)
             self.slider.setVisible(False)
         else:
@@ -423,33 +418,37 @@ class PageOutputSweep(page_uncertprop.PageOutput):
             super().outputupdate()
 
 
-class UncertSweepWidget(page_uncertprop.UncertPropWidget):
+class UncertSweepWidget(page_uncert.UncertPropWidget):
     ''' Uncertainty Propagation Sweep. Adds sweep tab to tabwidget '''
-    def __init__(self, item, parent=None):
-        assert isinstance(item, sweeper.UncertSweep)
-        self.uncSweep = item
-        self.sweepsetup = SweepSetupTable(self.uncSweep)
-        super().__init__(item.unccalc, parent)
+    def __init__(self, projitem, parent=None):
+        self.projitem = projitem
+        self.sweepsetup = SweepSetupTable(self.projitem)
+        super().__init__(projitem, parent)
 
         self.menu.removeAction(self.mnuSaveSamples.menuAction())
         self.actNewUnc = QtWidgets.QAction('New single calculation from sweep', self)
-        self.actNewUnc.triggered.connect(lambda event, x=item: self.newtype.emit(x.get_config(), 'uncertainty'))
+        self.actNewUnc.triggered.connect(lambda event, x=projitem: self.newtype.emit(x.get_config(), 'uncertainty'))
         self.actReverse.disconnect()
-        self.actReverse.triggered.connect(lambda event, x=item: self.newtype.emit(x.get_config(), 'reversesweep'))
+        self.actReverse.triggered.connect(lambda event, x=projitem: self.newtype.emit(x.get_config(), 'reversesweep'))
         self.menu.insertAction(self.actSweep, self.actNewUnc)
         self.menu.removeAction(self.actSweep)
         self.pgoutputsweep = PageOutputSweep(None)
         self.pgoutputsweep.back.connect(self.backbutton)
         self.stack.removeWidget(self.pgoutput)
         self.stack.addWidget(self.pgoutputsweep)
-        self.pginput.panel.insert_widget('Sweep', self.sweepsetup, 4)
+        _, buttons = self.pginput.panel.insert_widget('Sweep', self.sweepsetup, 4, buttons=True)
+        buttons.plusclicked.connect(self.sweepsetup.addcol)
+        buttons.minusclicked.connect(self.sweepsetup.remcol)
         self.pginput.panel.expand('Sweep')
+        self.update_proj_config()
+        self.sweepsetup.set_variables(self.projitem.model.model.variables)
+        self.pginput.funclist.funcchanged.connect(self.funcchanged)
+        self.pginput.meastable.changed.connect(self.funcchanged)
 
-    def funcchanged(self, row, fdict):
-        ''' Sweep function has changed '''
-        super().funcchanged(row, fdict)
-        baseinputs = self.uncSweep.unccalc.get_baseinputs()
-        self.sweepsetup.set_inptlist(baseinputs)
+    def funcchanged(self, funclist):
+        ''' Function has changed '''
+        self.update_proj_config()
+        self.sweepsetup.set_variables(self.projitem.model.model.variables)
         with suppress(AttributeError):  # May not be defined yet
             self.actNewUnc.setEnabled(True)
 
@@ -461,60 +460,96 @@ class UncertSweepWidget(page_uncertprop.UncertPropWidget):
 
     def calculate(self):
         ''' Run the calculation '''
-        valid = True
-        if not self.pginput.isValid():
-            QtWidgets.QMessageBox.warning(self, 'Uncertainty Calculator', 'Invalid Input Parameter!')
-            valid = False
-
-        elif len(self.uncSweep.unccalc.functions) < 1:
-            QtWidgets.QMessageBox.warning(self, 'Uncertainty Calculator', 'Need at least one measurement function to calculate.')
-            valid = False
-
-        elif len(self.uncSweep.sweeplist) < 1:
-            QtWidgets.QMessageBox.warning(self, 'Uncertainty Calculator', 'Please define at least one sweep in the Sweep tab.')
-            valid = False
-
-        if not valid:
+        def err_msg(msg):
+            QtWidgets.QMessageBox.warning(self, 'Suncal', msg)
             self.actSaveReport.setEnabled(False)
+            self.mnuSaveSamples.setEnabled(False)
+
+        if not (self.pginput.funclist.is_valid() and self.pginput.meastable.is_valid()):
+            err_msg('Invalid input parameter!')
+            return
+
+        config = self.get_config()
+        if len(config.get('functions', [])) < 1:
+            err_msg('No functions to compute!')
             return
 
         try:
-            self.uncSweep.calculate()
-        except (ValueError, RecursionError):
-            valid = False
+            self.update_proj_config()
+        except RecursionError:
+            err_msg('Circular reference in function definitions')
+            return
 
-        if valid:
-            self.pgoutputsweep.set_sweepreport(self.uncSweep.out)
-            self.pgoutputsweep.outputupdate()
-            self.stack.setCurrentIndex(self.PG_OUTPUT)
-            self.actSaveReport.setEnabled(True)
-        else:
-            QtWidgets.QMessageBox.warning(self, 'Uncertainty Calculator', 'Error computing solution!')
-            self.actSaveReport.setEnabled(False)
+        if len(self.projitem.model.sweeplist) < 1:
+            err_msg('Please define at least one sweep in the Sweep tab.')
+            return
+
+        elif len(set([len(s.get('values', [])) for s in self.projitem.model.sweeplist])) > 1:
+            err_msg('All sweep columns must be the same length.')
+            return
+
+        try:
+            self.projitem.model.model.eval()
+        except OffsetUnitCalculusError as exc:
+            badunit = re.findall(r'\((.+ )', str(exc))[0].split()[0].strip(', ')
+            err_msg(f'Ambiguous unit {badunit}. Try "delta_{badunit}".')
+            return
+        except (TypeError, DimensionalityError, UndefinedUnitError) as exc:
+            err_msg(f'Units Error: {exc}')
+            return
+        except RecursionError:
+            err_msg('Error - possible circular reference in function definitions')
+            return
+
+        try:
+            self.projitem.calculate()
+        except OffsetUnitCalculusError as exc:
+            badunit = re.findall(r'\((.+ )', str(exc))[0].split()[0].strip(', ')
+            err_msg(f'Ambiguous unit {badunit}. Try "delta_{badunit}".')
+            return
+        except RecursionError:
+            err_msg('Error - possible circular reference in function definitions')
+            return
+        except (TypeError, ValueError):
+            err_msg('Error computing solution!')
+            return
+
+        self.pgoutputsweep.set_sweepresult(self.projitem.result)
+        self.pgoutputsweep.outputupdate()
+        self.stack.setCurrentIndex(self.PG_OUTPUT)
+        self.actSaveReport.setEnabled(True)
+
+    def update_proj_config(self):
+        ''' Save page setup back to project item configuration '''
+        self.projitem.load_config(self.get_config())
+
+    def get_config(self):
+        config = self.pginput.get_config()
+        config.update({'sweeps': self.sweepsetup.get_config()})
+        return config
 
     def get_report(self):
         ''' Get full report of curve fit, using page settings '''
-        return self.uncSweep.get_output().report_all()
+        return self.projitem.result.report.all()
 
     def save_report(self):
         ''' Save full report, asking user for settings/filename '''
         gui_widgets.savereport(self.get_report())
 
 
-class UncertReverseSweepWidget(page_uncertprop.UncertPropWidget):
+class UncertReverseSweepWidget(page_uncert.UncertPropWidget):
     ''' Widget for calculating a reverse uncertainty sweep '''
-    def __init__(self, item, parent=None):
-        self.uncRevSwp = item
-        self.uncCalc = item.unccalc
-        self.sweepsetup = SweepSetupTable(self.uncRevSwp)
-        self.targetsetup = page_reverse.TargetSetupWidget(self.uncCalc)
-        super().__init__(self.uncCalc, parent=parent)
+    def __init__(self, projitem, parent=None):
+        self.projitem = projitem
+        self.sweepsetup = SweepSetupTable(self.projitem)
+        self.targetsetup = page_reverse.TargetSetupWidget(self.projitem)
+        super().__init__(self.projitem, parent=parent)
 
         self.menu.removeAction(self.mnuSaveSamples.menuAction())
         self.actNewRev = QtWidgets.QAction('New single reverse from sweep', self)
-        self.actNewRev.triggered.connect(lambda event, x=item: self.newtype.emit(x.get_config(), 'reverse'))
+        self.actNewRev.triggered.connect(lambda event, x=projitem: self.newtype.emit(x.get_config(), 'reverse'))
         self.actNewSwp = QtWidgets.QAction('New forward sweep from model', self)
-        self.actNewSwp.triggered.connect(lambda event, x=item: self.newtype.emit(x.get_config(), 'sweep'))
+        self.actNewSwp.triggered.connect(lambda event, x=projitem: self.newtype.emit(x.get_config(), 'sweep'))
         self.menu.insertAction(self.actSweep, self.actNewRev)
         self.menu.insertAction(self.actSweep, self.actNewSwp)
         self.menu.removeAction(self.actSweep)
@@ -523,14 +558,17 @@ class UncertReverseSweepWidget(page_uncertprop.UncertPropWidget):
         self.pgoutputrevsweep.back.connect(self.backbutton)
         self.stack.removeWidget(self.pgoutput)
         self.stack.addWidget(self.pgoutputrevsweep)
-        self.pginput.panel.insert_widget('Sweep', self.sweepsetup, 4)
+        _, buttons = self.pginput.panel.insert_widget('Sweep', self.sweepsetup, 4, buttons=True)
         self.pginput.panel.insert_widget('Reverse Target Value', self.targetsetup, 4)
+        buttons.plusclicked.connect(self.sweepsetup.addcol)
+        buttons.minusclicked.connect(self.sweepsetup.remcol)
+        self.pginput.funclist.funcchanged.connect(self.funcchanged)
+        self.pginput.meastable.changed.connect(self.funcchanged)
 
-    def funcchanged(self, row, fdict):
+    def funcchanged(self, funclist):
         ''' Function has changed '''
-        super().funcchanged(row, fdict)
-        baseinputs = self.uncCalc.get_baseinputs()
-        self.sweepsetup.set_inptlist(baseinputs)
+        self.update_proj_config()
+        self.sweepsetup.set_variables(self.projitem.model.model.variables)
         self.targetsetup.update_names()
         with suppress(AttributeError):
             self.actNewSwp.setEnabled(True)
@@ -545,43 +583,64 @@ class UncertReverseSweepWidget(page_uncertprop.UncertPropWidget):
 
     def calculate(self):
         ''' Run the calculation '''
-        valid = True
-        if not self.pginput.isValid():
-            QtWidgets.QMessageBox.warning(self, 'Uncertainty Calculator', 'Invalid Input Parameter!')
-            valid = False
-
-        elif len(self.uncCalc.functions) < 1:
-            QtWidgets.QMessageBox.warning(self, 'Uncertainty Calculator', 'Need at least one measurement function to calculate.')
-            valid = False
-
-        elif self.targetsetup.cmbSolveFor.currentText() == '':
-            QtWidgets.QMessageBox.warning(self, 'Uncertainty Calculator', 'Please define solve-for parameter in the Target tab.')
-            valid = False
-
-        elif len(self.uncRevSwp.sweeplist) < 1:
-            QtWidgets.QMessageBox.warning(self, 'Uncertainty Calculator', 'Please define at least one sweep in the Sweep tab.')
-            valid = False
-
-        if not valid:
+        def err_msg(msg):
+            QtWidgets.QMessageBox.warning(self, 'Suncal', msg)
             self.actSaveReport.setEnabled(False)
+            self.mnuSaveSamples.setEnabled(False)
+
+        if not (self.pginput.funclist.is_valid() and self.pginput.meastable.is_valid()):
+            err_msg('Invalid input parameter!')
+            return
+
+        self.update_proj_config()
+
+        if len(self.projitem.model.model.exprs) < 1:
+            err_msg('Need at least one measurement function to calculate.')
+            return
+
+        if self.targetsetup.cmbSolveFor.currentText() == '':
+            err_msg('Please define solve-for parameter in the Target tab.')
+            return
+
+        if len(self.projitem.model.sweeplist) < 1:
+            err_msg('Please define at least one sweep in the Sweep tab.')
+            return
+
+        if len({len(s.get('values', [])) for s in self.projitem.model.sweeplist}) > 1:
+            err_msg('All sweep columns must be the same length.')
             return
 
         try:
-            self.uncRevSwp.calculate()
+            self.projitem.calculate()
         except (ValueError, RecursionError):
-            valid = False
-
-        if valid:
-            self.pgoutputrevsweep.txtOutput.setReport(self.uncRevSwp.out.report_summary())
-            self.stack.setCurrentIndex(self.PG_OUTPUT)
-            self.actSaveReport.setEnabled(True)
-        else:
-            QtWidgets.QMessageBox.warning(self, 'Uncertainty Calculator', 'Error computing solution!')
+            QtWidgets.QMessageBox.warning(self, 'Suncal', 'Error computing solution!')
             self.actSaveReport.setEnabled(False)
+            return
+
+        self.pgoutputrevsweep.txtOutput.setReport(self.projitem.result.report.summary_withplots())
+        self.stack.setCurrentIndex(self.PG_OUTPUT)
+        self.actSaveReport.setEnabled(True)
+
+    def update_proj_config(self):
+        ''' Save page setup back to project item configuration '''
+        self.projitem.load_config(self.get_config())
+
+    def get_config(self):
+        config = self.pginput.get_config()
+        config.update({'sweeps': self.sweepsetup.get_config()})
+
+        reverseparams = self.targetsetup.get_target()
+        functions = config.get('functions', [])
+        funcnames = [f['name'] for f in functions]
+        function = functions[funcnames.index(reverseparams['funcname'])]
+        units = function.get('units', None)
+        reverseparams['targetunits'] = units
+        config.update({'reverse': reverseparams}) 
+        return config
 
     def get_report(self):
         ''' Get full report of curve fit, using page settings '''
-        return self.uncRevSwp.get_output().report_all()
+        return self.projitem.result.report.all()
 
     def save_report(self):
         ''' Save full report, asking user for settings/filename '''
